@@ -6,6 +6,7 @@ import uniplot.layer_assembly as layer_assembly
 import uniplot.plot_elements as elements
 from uniplot.getch import getch
 from uniplot.param_initializer import validate_and_transform_options
+from uniplot.axis_labels.extended_talbot_labels import extended_talbot_labels
 
 
 def plot(ys: Any, xs: Optional[Any] = None, **kwargs) -> None:
@@ -35,13 +36,27 @@ def plot(ys: Any, xs: Optional[Any] = None, **kwargs) -> None:
         if not options.interactive:
             continue_looping = False
 
-        # Prepare plot elements
-        y_axis_labels = elements.yaxis_ticks(
-            y_min=options.y_min, y_max=options.y_max, height=options.height
+        # Prepare y axis labels
+        y_axis_labels = ["-error generating labels, sorry-"] * options.height
+        y_axis_label_set = extended_talbot_labels(
+            x_min=options.y_min,
+            x_max=options.y_max,
+            available_space=options.height,
+            vertical_direction=True,
         )
-        x_axis_labels = elements.xaxis_ticks(
-            x_min=options.x_min, x_max=options.x_max, width=options.width
+        if y_axis_label_set is not None:
+            y_axis_labels = y_axis_label_set.render()
+
+        # Prepare x axis labels
+        x_axis_label_set = extended_talbot_labels(
+            x_min=options.x_min,
+            x_max=options.x_max,
+            available_space=options.width,
+            vertical_direction=False,
         )
+        x_axis_labels = "-error generating labels, sorry-"
+        if x_axis_label_set is not None:
+            x_axis_labels = x_axis_label_set.render()[0]
 
         # Prefare graph surface
         pixel_character_matrix = layer_assembly.assemble_scatter_plot(
@@ -116,3 +131,52 @@ def plot(ys: Any, xs: Optional[Any] = None, **kwargs) -> None:
                 continue_looping = False
 
             loop_iteration += 1
+
+
+def histogram(
+    xs: Any,
+    bins: int = 20,
+    bins_min: Optional[float] = None,
+    bins_max: Optional[float] = None,
+    **kwargs,
+) -> None:
+    """
+    Plot a histogram to the terminal.
+
+    Parameters:
+
+    - `xs` are the values of the points to plot. This parameter is mandatory and
+      can either be a list or a list of lists, or the equivalent NumPy array.
+    - Any additional keyword arguments are passed to the `uniplot.options.Options` class.
+    """
+    # HACK Use the `MultiSeries` constructor to cast values to uniform format
+    multi_series = MultiSeries(ys=xs)
+
+    # Histograms usually make sense only with lines
+    kwargs["lines"] = kwargs.get("lines", True)
+
+    bins_min = bins_min or multi_series.y_min()
+    bins_max = bins_max or multi_series.y_max()
+    range = bins_max - bins_min
+    if range > 0:
+        bins_min = bins_min - 0.1 * range
+        bins_max = bins_max + 0.1 * range
+
+    xs_histo_series = []
+    ys_histo_series = []
+    for s in multi_series.ys:
+        hist, bin_edges = np.histogram(s, bins=bins, range=(bins_min, bins_max))
+        xs_here = np.zeros(1 + 2 * bins + 1)
+        ys_here = np.zeros(1 + 2 * bins + 1)
+        xs_here[0] = bin_edges[0]
+        xs_here[1::2] = bin_edges
+        xs_here[2::2] = bin_edges[1:]
+        # ys_here[0] = 0
+        ys_here[1:-1:2] = hist
+        ys_here[2:-1:2] = hist
+        # ys_here[-1] = 0
+
+        xs_histo_series.append(xs_here)
+        ys_histo_series.append(ys_here)
+
+    plot(xs=xs_histo_series, ys=ys_histo_series, **kwargs)
